@@ -9,6 +9,7 @@ import sys
 from contextlib import contextmanager
 from inspect import Parameter
 from inspect import signature
+from typing import overload
 
 import attr
 import py
@@ -27,9 +28,9 @@ MODULE_NOT_FOUND_ERROR = (
 
 
 if sys.version_info >= (3, 8):
-    from importlib import metadata as importlib_metadata  # noqa
+    from importlib import metadata as importlib_metadata  # noqa: F401
 else:
-    import importlib_metadata  # noqa
+    import importlib_metadata  # noqa: F401
 
 
 def _format_args(func):
@@ -347,3 +348,9 @@ class FuncargnamesCompatAttr:
 
         warnings.warn(FUNCARGNAMES, stacklevel=2)
         return self.fixturenames
+
+
+if sys.version_info < (3, 5, 2):  # pragma: no cover
+
+    def overload(f):  # noqa: F811
+        return f
