@@ -55,3 +55,36 @@ def pytest_collection_modifyitems(config, items):
     items[:] = fast_items + neutral_items + slow_items + slowest_items
 
     yield
+
+
+@pytest.fixture
+def tw_mock():
+    """Returns a mock terminal writer"""
+
+    class TWMock:
+        WRITE = object()
+
+        def __init__(self):
+            self.lines = []
+            self.is_writing = False
+
+        def sep(self, sep, line=None):
+            self.lines.append((sep, line))
+
+        def write(self, msg, **kw):
+            self.lines.append((TWMock.WRITE, msg))
+
+        def line(self, line, **kw):
+            self.lines.append(line)
+
+        def markup(self, text, **kw):
+            return text
+
+        def get_write_msg(self, idx):
+            flag, msg = self.lines[idx]
+            assert flag == TWMock.WRITE
+            return msg
+
+        fullwidth = 80
+
+    return TWMock()
