@@ -1,3 +1,4 @@
+import os.path
 import textwrap
 
 import py
@@ -5,6 +6,7 @@ import py
 import pytest
 from _pytest.config import PytestPluginManager
 from _pytest.main import ExitCode
+from _pytest.pathlib import unique_path
 
 
 def ConftestWithSetinitial(path):
@@ -141,11 +143,11 @@ def test_conftestcutdir(testdir):
     # but we can still import a conftest directly
     conftest._importconftest(conf)
     values = conftest._getconftestmodules(conf.dirpath())
-    assert values[0].__file__.startswith(str(conf))
+    assert values[0].__file__.startswith(str(unique_path(conf)))
     # and all sub paths get updated properly
     values = conftest._getconftestmodules(p)
     assert len(values) == 1
-    assert values[0].__file__.startswith(str(conf))
+    assert values[0].__file__.startswith(str(unique_path(conf)))
 
 
 def test_conftestcutdir_inplace_considered(testdir):
@@ -154,7 +156,7 @@ def test_conftestcutdir_inplace_considered(testdir):
     conftest_setinitial(conftest, [conf.dirpath()], confcutdir=conf.dirpath())
     values = conftest._getconftestmodules(conf.dirpath())
     assert len(values) == 1
-    assert values[0].__file__.startswith(str(conf))
+    assert values[0].__file__.startswith(str(unique_path(conf)))
 
 
 @pytest.mark.parametrize("name", "test tests whatever .dotdir".split())
@@ -164,7 +166,7 @@ def test_setinitial_conftest_subdirs(testdir, name):
     conftest = PytestPluginManager()
     conftest_setinitial(conftest, [sub.dirpath()], confcutdir=testdir.tmpdir)
     if name not in ("whatever", ".dotdir"):
-        assert subconftest in conftest._conftestpath2mod
+        assert unique_path(subconftest) in conftest._conftestpath2mod
         assert len(conftest._conftestpath2mod) == 1
     else:
         assert subconftest not in conftest._conftestpath2mod
@@ -275,6 +277,21 @@ def test_conftest_symlink_files(testdir):
     assert result.ret == ExitCode.OK
 
 
+@pytest.mark.skipif(
+    os.path.normcase("x") != os.path.normcase("X"),
+    reason="only relevant for case insensitive file systems",
+)
+def test_conftest_badcase(testdir):
+    """Check conftest.py loading when directory casing is wrong."""
+    testdir.tmpdir.mkdir("JenkinsRoot").mkdir("test")
+    source = {"setup.py": "", "test/__init__.py": "", "test/conftest.py": ""}
+    testdir.makepyfile(**{"JenkinsRoot/%s" % k: v for k, v in source.items()})
+
+    testdir.tmpdir.join("jenkinsroot/test").chdir()
+    result = testdir.runpytest()
+    assert result.ret == ExitCode.NO_TESTS_COLLECTED
+
+
 def test_no_conftest(testdir):
     testdir.makeconftest("assert 0")
     result = testdir.runpytest("--noconftest")
