@@ -1,12 +1,7 @@
-# -*- coding: utf-8 -*-
 """
  test correct setup/teardowns at
  module, class, and instance level
 """
-from __future__ import absolute_import
-from __future__ import division
-from __future__ import print_function
-
 import pytest
 
 
